/************************************************************************
*                                                                       *
*   Filename:      MC_L3-Flash+PB_LED-int-PCL.c                         *
*   Date:          5/7/09                                               *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 3, example 2                                 *
*                                                                       *
*   Demonstrates use of Timer0 interrupt to perform a background task   *
*   while performing other actions in repsonse to changing inputs       *
*                                                                       *
*   One LED simply flashes at 1 Hz (50% duty cycle).                    *
*   The other LED is only lit when the pushbutton is pressed.           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP1 - "button pressed" indicator LED                            *
*       GP2 - flashing LED                                              *
*       GP3 - pushbutton                                                *
*                                                                       *
************************************************************************/

#include <htc.h>

/***** CONFIGURATION *****/
//      int reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLRDIS & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nB_LED  1       // "button pressed" indicator LED on GP1
#define nF_LED  2       // flashing LED on GP2
#define BUTTON  GPIO3   // pushbutton on GP3


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // setup ports
    GPIO = 0;                           // start with LEDs off
    sGPIO = 0;                          //   update shadow
    TRISIO = ~(1<<nB_LED|1<<nF_LED);    // configure LED pins (only) as outputs
    // configure Timer0
    T0CS = 0;                           // select timer mode
    PSA = 1;                            // no prescaler (assigned to WDT)
                                        //  -> increment TMR0 every 1 us             
    // configure interrupts
    T0IE = 1;                           // enable Timer0 interrupt
    ei();                               // enable global interrupts
                                    
    // Main loop
    for (;;) {
        // check and respond to button press
        sGPIO &= ~(1<<nB_LED);          // assume button up -> LED off
        if (BUTTON == 0)                // if button pressed (GP3 low)
            sGPIO |= 1<<nB_LED;         //   turn on indicator LED

        // continually copy shadow GPIO to port
        GPIO = sGPIO; 
            
    }   // repeat forever
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    static unsigned int     cnt_t0 = 0;     // counts timer0 overflows
    
    // service Timer0 interrupt
    //   TMR0 overflows every 250 clocks = 250 us
    //   (only Timer0 interrupts are enabled)
    //
    TMR0 += 256-250+3;              // add value to Timer0
                                    //   for overflow after 250 counts
    T0IF = 0;                       // clear interrupt flag
    
    // toggle LED every 500 ms
    ++cnt_t0;                       // increment interrupt count (every 250 us)
    if (cnt_t0 == 500000/250) {     // if count overflow (every 500 ms),
        cnt_t0 = 0;                 //   reset count
        sGPIO ^= 1<<nF_LED;         //   toggle LED (using shadow register)
    }
}
