/************************************************************************
*                                                                       *
*   Filename:      MC_L3-Flash+Toggle_LED-PCL.c                         *
*   Date:          5/7/09                                               *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 3, example 4                                 *
*                                                                       *
*   Demonstrates use of Timer0 interrupt to run multiple                *
*   concurrent background tasks                                         *
*                                                                       *
*   Toggles LED when the pushbutton is pressed                          *
*   while another LED flashes at 1 Hz (50% duty cycle)                  *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP1 - "button pressed" indicator LED                            *
*       GP2 - flashing LED                                              *
*       GP3 - pushbutton (active low)                                   *
*                                                                       *
************************************************************************/

#include <htc.h>

/***** CONFIGURATION *****/
//      int reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLRDIS & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nB_LED  1           // "button pressed" indicator LED on GP1
#define nF_LED  2           // flashing LED on GP2
#define BUTTON  GPIO3       // pushbutton on GP3 (active low)


/***** CONSTANTS *****/
#define MAX_DB_CNT  20/2    // maximum debounce count = debounce period / sample rate
                            //   (20 ms debounce period / 2 ms per sample)


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO
bit             PB_dbstate;     // debounced pushbutton state (1 = released)
bit             PB_change;      // pushbutton state change flag (1 = changed)


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // setup ports
    GPIO = 0;                           // start with LEDs off
    sGPIO = 0;                          //   update shadow
    TRISIO = ~(1<<nB_LED|1<<nF_LED);    // configure LED pins as outputs
    // configure Timer0
    T0CS = 0;                           // select timer mode
    PSA = 1;                            // no prescaler (assigned to WDT)
                                        //  -> increment TMR0 every 1 us
    // initialise variables
    PB_dbstate = 1;                     // initial pushbutton state = released
    PB_change = 0;                      // clear pushbutton change flag (no change)
    // configure interrupts
    T0IE = 1;                           // enable Timer0 interrupt
    ei();                               // enable global interrupts
                                    
    // Main loop
    for (;;)
    {
        // check for debounced button press
        if (PB_change && !PB_dbstate)       // if button state changed and pressed (low)
        {
            sGPIO ^= 1<<nB_LED;             //   toggle indicator LED (using shadow)
            PB_change = 0;                  //   clear button change flag
        }
        // continually copy shadow GPIO to port
        GPIO = sGPIO; 
            
    }   // repeat forever
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    static unsigned char    db_t_cnt = 0;   // debounce sample timebase counter
    static unsigned char    db_s_cnt = 0;   // debounce sample counter
    static unsigned int     fl_t_cnt = 0;   // LED flash timebase counter
    
    // service Timer0 interrupt
    //   TMR0 overflows every 250 clocks = 250 us
    //   (only Timer0 interrupts are enabled)
    //   
    TMR0 += 256-250+3;              // add value to Timer0
                                    //   for overflow after 250 counts
    T0IF = 0;                       // clear interrupt flag
    
    // Debounce pushbutton (every 2 ms)
    //   use counting algorithm: accept change in state
    //   only if new state is seen a number of times in succession
    //
    // sample switch every 2 ms (8 interrupts x 250 us)
    ++db_t_cnt;                     // increment interrupt count (every 250 us)
    if (db_t_cnt == 2000/250)       // until 2 ms has elapsed
    {
        db_t_cnt = 0;               //   reset interrupt count
        
        // compare raw pushbutton with current debounced state
        if (BUTTON == PB_dbstate)   // if raw pushbutton matches current debounce state,
        {
            db_s_cnt = 0;           //   reset debounce count
        }
        else                        // else raw pushbutton has changed state
        {
            ++db_s_cnt;                     // increment debounce count
            if (db_s_cnt == MAX_DB_CNT)     // when max count is reached
            {                               //   accept new state as changed:
                PB_dbstate = !PB_dbstate;   //     toggle debounced state
                db_s_cnt = 0;               //     reset debounce count
                PB_change = 1;              //     set pushbutton changed flag
                                            //     (polled and cleared in main loop)
            }
        }   
    }
    
    // Flash LED (toggle every 500 ms)
    //
    ++fl_t_cnt;                     // increment interrupt count (every 250 us)
    if (fl_t_cnt == 500000/250)     // until 500 ms has elapsed
    {
        fl_t_cnt = 0;               //   reset interrupt count
        sGPIO ^= 1<<nF_LED;         //   toggle LED (using shadow register)
    }
}
