/************************************************************************
*                                                                       *
*   Filename:      MC_L3-Flash+Toggle_LED-ext_int-PCL.c                 *
*   Date:          5/7/09                                               *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 3, example 6                                 *
*                                                                       *
*   Demonstrates handling of multiple interrupt sources                 *
*                                                                       *
*   Toggles LED on GP1 when pushbutton on INT is pressed                *
*   (high -> low transition triggering external interrupt)              *
*   while LED on GP0 flashes at 1 Hz (driven by Timer0 interrupt)       *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP0 - flashing LED                                              *
*       GP1 - indicator LED                                             *
*       INT - pushbutton (active low)                                   *
*                                                                       *
************************************************************************/

#include <htc.h>

/***** CONFIGURATION *****/
//      ext reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nF_LED  0               // flashing LED on GP0
#define nB_LED  1               // indicator LED on GP1


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // setup ports
    GPIO = 0;                           // start with LEDs off
    sGPIO = 0;                          //   update shadow
    TRISIO = ~(1<<nB_LED|1<<nF_LED);    // configure LED pins as output
    // configure Timer0
    T0CS = 0;                           // select timer mode
    PSA = 1;                            // no prescaler (assigned to WDT)
                                        //  -> increment TMR0 every 1 us
    // configure external interrupt
    INTEDG = 0;                         // trigger on falling edge 
    // configure interrupts
    T0IE = 1;                           // enable Timer0 interrupt
    INTE = 1;                           // enable external interrupt
    ei();                               // enable global interrupts
                                    
    // Main loop
    for (;;)
    {
        // continually copy shadow GPIO to port
        GPIO = sGPIO; 
            
    }   // repeat forever
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    static unsigned int     fl_t_cnt = 0;   // LED flash timebase counter
    
    // Service all triggered interrupt sources
    
    if (INTF)
    {
        // External interrupt
        //   Triggered on high -> low transition on INT pin
        //   caused by externally debounced pushbutton press
        //   
        INTF = 0;                       // clear interrupt flag
    
        // toggle LED
        sGPIO ^= 1<<nB_LED;             // only update shadow register    
    }
    
    if (T0IF)
    {
        // Timer0 interrupt
        //   TMR0 overflows every 250 clocks = 250 us
        //   (only Timer0 interrupts are enabled)
        //   
        TMR0 += 256-250+3;              // add value to Timer0
                                        //   for overflow after 250 counts
        T0IF = 0;                       // clear interrupt flag
        
        // Flash LED (toggle every 500 ms)
        //
        ++fl_t_cnt;                     // increment interrupt count (every 250 us)
        if (fl_t_cnt == 500000/250)     // until 500 ms has elapsed
        {
            fl_t_cnt = 0;               //   reset interrupt count
            sGPIO ^= 1<<nF_LED;         //   toggle LED (using shadow register)
        }
    }
}
