/************************************************************************
*                                                                       *
*   Filename:      MC_L3-Toggle_LED-count-int-PCL.c                     *
*   Date:          5/7/09                                               *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 3, example 3                                 *
*                                                                       *
*   Demonstrates use of Timer0 interrupt to implement                   *
*   counting debounce algorithm                                         *
*                                                                       *
*   Toggles LED when the pushbutton is pressed (high -> low)            *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP1 - indicator LED                                             *
*       GP3 - pushbutton (active low)                                   *
*                                                                       *
************************************************************************/

#include <htc.h>

/***** CONFIGURATION *****/
//      int reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLRDIS & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nB_LED  1           // indicator LED on GP1
#define BUTTON  GPIO3       // pushbutton on GP3 (active low)


/***** CONSTANTS *****/
#define MAX_DB_CNT  20/2    // maximum debounce count = debounce period / sample rate
                            //   (20 ms debounce period / 2 ms per sample)


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO
bit             PB_dbstate;     // debounced pushbutton state (1 = released)
bit             PB_change;      // pushbutton state change flag (1 = changed)


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // setup ports
    GPIO = 0;                       // start with LED off
    sGPIO = 0;                      //   update shadow
    TRISIO = ~(1<<nB_LED);          // configure LED pin as output
    // configure Timer0
    T0CS = 0;                       // select timer mode
    PSA = 1;                        // no prescaler (assigned to WDT)
                                    //  -> increment TMR0 every 1 us 
    // initialise variables
    PB_dbstate = 1;                 // initial pushbutton state = released
    PB_change = 0;                  // clear pushbutton change flag (no change)
    // configure interrupts
    T0IE = 1;                       // enable Timer0 interrupt
    ei();                           // enable global interrupts
                                    
    // Main loop
    for (;;)
    {
        // check for debounced button press
        if (PB_change && !PB_dbstate)       // if button state changed and pressed (low)
        {
            sGPIO ^= 1<<nB_LED;             //   toggle indicator LED (using shadow)
            PB_change = 0;                  //   clear button change flag
        }
        // continually copy shadow GPIO to port
        GPIO = sGPIO; 
            
    }   // repeat forever
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    static unsigned char    t0_cnt = 0;     // counts timer0 overflows
    static unsigned char    db_cnt = 0;     // debounce counter
    
    // service Timer0 interrupt
    //   TMR0 overflows every 250 clocks = 250 us
    //   (only Timer0 interrupts are enabled)
    //   
    TMR0 += 256-250+3;              // add value to Timer0
                                    //   for overflow after 250 counts
    T0IF = 0;                       // clear interrupt flag
    
    // Debounce pushbutton (every 2 ms)
    //   use counting algorithm: accept change in state
    //   only if new state is seen a number of times in succession
    
    // sample switch every 2 ms (8 interrupts x 250 us)
    ++t0_cnt;                       // increment interrupt count (every 250 us)
    if (t0_cnt == 2000/250)         // until 2 ms has elapsed
    {
        t0_cnt = 0;                 //   reset interrupt count
        
        // compare raw pushbutton with current debounced state
        if (BUTTON == PB_dbstate)   // if raw pushbutton matches current debounce state,
        {
            db_cnt = 0;             //   reset debounce count
        }
        else                        // else raw pushbutton has changed state
        {
            ++db_cnt;                   // increment debounce count
            if (db_cnt == MAX_DB_CNT)   // when max count is reached
            {                               // accept new state as changed:
                PB_dbstate = !PB_dbstate;   //   toggle debounced state
                db_cnt = 0;                 //   reset debounce count
                PB_change = 1;              //   set pushbutton changed flag
                                            //   (polled and cleared in main loop)
            }
        }   
    }
}
