/************************************************************************
*                                                                       *
*   Filename:      MC_L4-2xToggle_LED-IOC-PCL.c                         *
*   Date:          6/7/09                                               *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 4, example 2                                 *
*                                                                       *
*   Demonstrates use of interrupt-on-change interrupts                  *
*   (without software debouncing)                                       *
*                                                                       *
*   Toggles LED on GP0 when pushbutton on GP2 is pressed                *
*   (high -> low transition)                                            *
*   and LED on GP1 when pushbutton on GP4 is pressed                    *
*   (low -> high transition)                                            *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP0 - indicator LED 1                                           *
*       GP1 - indicator LED 2                                           *
*       GP2 - pushbutton 1 (externally debounced, active low)           *
*       GP4 - pushbutton 2 (externally debounced, active high)          *
*                                                                       *
************************************************************************/

#include <htc.h>

/***** CONFIGURATION *****/
//      ext reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nB1_LED 0           // "button 1 pressed" indicator LED on GP0
#define nB2_LED 1           // "button 2 pressed" indicator LED on GP1
#define nPB1    2           // pushbutton 1 (ext debounce, active low) on GP2
#define nPB2    4           // pushbutton 2 (ext debounce, active high) on GP4


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO
unsigned char   lGPIO;          // last state of GPIO (for change detection)


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // configure and initialise ports
    TRISIO = ~(1<<nB1_LED|1<<nB2_LED);  // configure LED pins as outputs
    IOCB |= 1<<nPB1|1<<nPB2;            // enable IOC on pushbuttons 1 and 2
    GPIO = 0;                   // start with all LEDs off
    sGPIO = 0;                  //   update shadow
    lGPIO = GPIO;               // update last state, for pin change detection
                                //   (GPIO read will clear any IOC mismatch)    
    
    // configure interrupts
    GPIE = 1;                   // enable port change interrupt
    ei();                       // enable global interrupts
                                    
    // Main loop
    for (;;)
    {
        // continually copy shadow GPIO to port
        GPIO = sGPIO; 
            
    }   // repeat forever
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    unsigned char   cGPIO;      // current state of GPIO (used by IOC handler)
    
    // Service interrupt-on-change
    //   Triggered on any transition on IOC-enabled input pin
    //   caused by externally debounced pushbutton press
    //   
    GPIF = 0;                       // clear interrupt flag
    
    // determine which pins have changed
    cGPIO = GPIO;                   // save current GPIO state
                                    //   (GPIO read clears IOC mismatch condition)
    lGPIO ^= cGPIO;                 // XOR current with last state to detect changes
    
    // toggle LED 1 only on button 1 press (active low)
    if ((lGPIO & 1<<nPB1)           // if button 1 changed
        && (!(cGPIO & 1<<nPB1)))    //  and button 1 is down (low)
        {         
            sGPIO ^= 1<<nB1_LED;    //   toggle LED 1 using shadow register
        }
        
    // toggle LED 2 only on button 2 press (active high)
    if ((lGPIO & 1<<nPB2)           // if button 2 changed
        && (cGPIO & 1<<nPB2))       //  and button 2 is down (high)
        {         
            sGPIO ^= 1<<nB2_LED;    //   toggle LED 2 using shadow register
        }    
    
    // update last GPIO state (for next time)
    lGPIO = cGPIO;                  // new "last state" = current
}
