/************************************************************************
*                                                                       *
*   Filename:      MC_L4-Toggle_LED-IOC-PCL.c                           *
*   Date:          30/8/09                                              *
*   File Version:  1.2                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 4, example 1                                 *
*                                                                       *
*   Demonstrates use of interrupt-on-change interrupts                  *
*   (without software debouncing)                                       *
*                                                                       *
*   Toggles LED on GP1                                                  *
*   when pushbutton on GP2 is pressed (high -> low transition)          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP1 - indicator LED                                             *
*       GP2 - pushbutton (externally debounced, active low)             *
*                                                                       *
************************************************************************/

#include <htc.h>

/***** CONFIGURATION *****/
//      ext reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nB_LED  1               // indicator LED on GP1
#define nBUTTON 2               // externally debounced pushbutton on GP2


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // setup ports
    TRISIO = ~(1<<nB_LED);      // configure LED pin (only) as output
    IOCB |= 1<<nBUTTON;         // enable IOC on pushbutton input
    GPIO = 0;                   // start with all LEDs off
                                //   (write to GPIO will clear any IOC mismatch)
    sGPIO = 0;                  //   update shadow
    
    // configure interrupts
    GPIE = 1;                   // enable port change interrupt
    ei();                       // enable global interrupts
                                    
    // Main loop
    for (;;)
    {
        // continually copy shadow GPIO to port
        GPIO = sGPIO; 
            
    }   // repeat forever
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    // Service interrupt-on-change
    //   Triggered on any transition on IOC-enabled input pin
    //   caused by externally debounced pushbutton press
    //   
    GPIO;                           // read GPIO to clear mismatch condition
    GPIF = 0;                       // clear interrupt flag
    
    // toggle LED only on button press
    if (!(GPIO & 1<<nBUTTON))       // if button is down
    {         
        sGPIO ^= 1<<nB_LED;         //   toggle LED using shadow register
    }
}
