/************************************************************************
*                                                                       *
*   Filename:      MC_L4-Wakeup-PCL.c                                   *
*   Date:          9/4/09                                               *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: dbmacros-PCL.h     (provides debounce macros)       *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 4, example 4                                 *
*                                                                       *
*   Demonstrates use of interrupt-on-change for wake-up from sleep mode *
*                                                                       *
*   Toggle LED, sleeping while LED is off:                              *
*       Turn on LED, debounce pushbutton then wait for button press     *
*       turn off LED, debounce pushbutton, sleep then repeat            *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP1 - on/off indicator LED                                      *
*       GP3 - pushbutton (active low)                                   *
*                                                                       *
************************************************************************/

#include <htc.h>

#include "dbmacros-PCL.h"   // DbnceHi() - debounce switch, wait for high
                            //           - uses TMR0 at 256us/tick


/***** CONFIGURATION *****/
//      int reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLRDIS & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define LED     GPIO1           // on/off indicator LED on GP1
#define nLED    1               //   (pin #1)
#define BUTTON  GPIO3           // pushbutton on GP3 (active low)
#define nBUTTON 3               //   (pin #3)


/***** MAIN PROGRAM *****/
void main()
{
    /*** Initialisation ***/
    // configure port
    TRISIO = ~(1<<nLED);        // configure LED pin (only) as output

    // configure Timer0 (for DbnceHi() macro)
    OPTION = 0b11010111;            // configure Timer0:
             //--0-----                 timer mode (T0CS = 0)
             //----0---                 prescaler assigned to Timer0 (PSA = 0)
             //-----111                 prescale = 256 (PS = 111)
                                    //  -> increment every 256 us
    // configure interrupt-on-change
    IOCB |= 1<<nBUTTON;         // enable IOC on pushbutton input
    GPIE = 1;                   // enable wake-up (interrupt) on port change

    
    /*** Main loop ***/
    for (;;)
    {
        LED = 1;                    // turn on LED
    
        DbnceHi(BUTTON);            // wait for stable button high
                                    //   (in case restarted after button press)
    
        while (BUTTON == 1)         // wait for button press (low)
            ;

        LED = 0;                    // turn off LED
    
        DbnceHi(BUTTON);            // wait for stable button release
        
        GPIF = 0;                   // clear port change interrupt flag
        
        SLEEP();                    // enter sleep mode
    }
}
