/************************************************************************
*                                                                       *
*   Filename:      MC_L5-POR+BODdemo-HTC.c                              *
*   Date:          12/7/09                                              *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH C PRO v9.65PL1 Lite                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 5, example 6                                 *
*                                                                       *
*   Demonstrates use of brown-out detect                                *
*   and differentiation between POR, BOD and MCLR resets                *
*                                                                       *
*   Turns on POR LED only if power-on reset is detected                 *
*   Turns on BOD LED only if brown-out detect reset is detected         *
*   Turns on indicator LED in all cases                                 *
*   (no POR or BOD implies MCLR, as no other reset sources are active)  *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       GP0 - "on" indicator LED (always turned on)                     *
*       GP1 - POR LED (indicates power-on reset)                        *
*       GP2 - BOD LED (indicates brown-out detected)                    *
*                                                                       *
************************************************************************/

#include <htc.h>


/***** CONFIGURATION *****/
// Config: ext reset, no code or data protect, brownout detect,
//         no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLREN & UNPROTECT & BOREN & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nO_LED  0           // "on" indicator LED on GP0 (always on)
#define nP_LED  1           // POR LED on GP1 to indicate power-on reset
#define nB_LED  2           // BOD LED on GP2 to indicate brown-out


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO


/***** MAIN PROGRAM *****/
void main()
{
    /*** Initialisation ***/
    // configure port
    TRISIO = ~(1<<nO_LED|1<<nP_LED|1<<nB_LED);  // configure LED pins as outputs
    
    // initialise port
    GPIO = 0;                       // start with all LEDs off
    sGPIO = 0;                      //   update shadow

    // check for POR or BOD reset
    if (!POR)                   // if power-on reset (*POR = 0),
    {
        POR = 1;                //   set POR and BOD flags for next reset
        BOD = 1;
        sGPIO |= 1<<nP_LED;     //   turn on POR LED (shadow)
    }
    if (!BOD)                   // if brown-out detect (*BOD = 0)
    {
        BOD = 1;                //   set BOD flag for next reset
        sGPIO |= 1<<nB_LED;     //   turn on BOD LED (shadow)   
    }     
    
    /*** Main code ***/
    sGPIO |= 1<<nO_LED;         // turn on "on" indicator LED (shadow)

    GPIO = sGPIO;               // copy shadow to GPIO
    
    for (;;)                    // wait forever
        ;
}
