/************************************************************************
*                                                                       *
*   Filename:      MC_L6-2xComp_Wakeup-PCL.c                            *
*   Date:          4/11/09                                              *
*   File Version:  1.2                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: delay.h     : provides DelayUs() macro              *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 6, example 10                                *
*                                                                       *
*   Demonstrates wake-up on dual comparator change                      *
*                                                                       *
*   Device wakes when input crosses a low or high threshold             * 
*                                                                       *
*   Turns on Low  LED when C2IN+ < C1IN- (1.1 V)                        *
*         or High LED when C2IN+ > C2IN- (2.0 V)                        *
*   then sleeps until the next change                                   *
*                                                                       *
*   (thresholds are set by external voltage dividers)                   *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       C2IN+ - voltage to be measured (e.g. pot output or LDR)         *
*       C1IN- - low threshold (1.1 V)                                   *
*       C2IN- - high threshold (2.0 V)                                  *
*       RC3   - "Low" lED                                               *
*       RC2   - "High" LED                                              *
*                                                                       *
************************************************************************/

#include <htc.h>

#define XTAL_FREQ   4MHZ    // oscillator frequency for DelayUs()
#include "delay.h"          // defines DelayUs()


/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO & FCMDIS & IESODIS);

// Pin assignments
#define LO      RC3         // "Low" LED
#define nLO     3           //   (bit 3)
#define HI      RC2         // "High" LED
#define nHI     2           //   (bit 2)


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    
    // configure ports
    TRISC = ~(1<<nLO|1<<nHI);   // configure LED pins (only) as outputs
    
    // configure comparators
    CMCON0 = 0b011;             // select mode 3 (CM = 011):
                                //   C1 -ref is C1IN-,
                                //   C1 +ref is C2IN+,
                                //   C2 -ref is C2IN-,
                                //   C2 +ref is C2IN+,
                                //   no external outputs,
                                //   both comparators on
    C1INV = 1;                  // C1 output inverted
                                //  -> C1OUT = 1 if C2IN+ < C1IN-
    C2INV = 0;                  // C2 output not inverted 
                                //  -> C2OUT = 1 if C2IN+ > C2IN-
 
    // enable comparator interrupts (for wake on change)
    PEIE = 1;                   // enable peripheral interrupts
    C1IE = 1;                   // and comparator interrupts
    C2IE = 1;

                                    
    // Main loop
    for (;;)
    {
        // test for and wait while input low
        while (C1OUT)           // while C2IN+ < C1IN- (1.1 V)
        {
            LO = 1;             //   turn on "low" LED
            DelayUs(2);         //   short delay to settle
        }

        // test for and wait while input high
        while (C2OUT)           // while C2IN+ > C2IN- (2.0 V)
        {
            HI = 1;             //   turn on "high" LED
            DelayUs(2);         //   short delay to settle
        }
            
        // turn off LEDs
        PORTC = 0;
        
        // enter sleep mode
        CMCON0;                 // read CMCON0 to clear comparator mismatch
        C1IF = 0;               // clear comparator interrupt flags
        C2IF = 0;
        SLEEP();
        
        // repeat forever
    }
}

