/************************************************************************
*                                                                       *
*   Filename:      MC_L6-2xComp_interrupt-PCL.c                         *
*   Date:          27/10/09                                             *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 6, example 9                                 *
*                                                                       *
*   Demonstrates use of interrupts with two comparators                 *
*   (assumes hysteresis is used to reduce triggering)                   *
*                                                                       *
*   Turns on: LED 1 when C1IN- > C2IN+,                                 *
*         and LED 2 when C2IN- > C2IN+                                  *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       C1IN- - input 1 (LDR/resistor divider)                          *
*       C2IN- - input 2 (LDR/resistor divider)                          *
*       C2IN+ - common reference (LDR/resistor divider)                 *
*       RC2   - indicator LED 1                                         *
*       RC3   - indicator LED 2                                         *
*                                                                       *
************************************************************************/

#include <htc.h>


/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO & FCMDIS & IESODIS);

// Pin assignments
#define nLED1   2               // indicator LED 1 on RC2
#define nLED2   3               // indicator LED 2 on RC3


/***** GLOBAL VARIABLES *****/
unsigned char   sPORTC;         // shadow copy of PORTC


/***** MAIN PROGRAM *****/
void main()
{
    /*** Initialisation ***/
    
    // configure ports
    TRISC = 0b000011;           // configure comparator input pins on PORTC
                                //   (RC0/C2IN+ and RC1/C2IN-) as inputs

    // configure comparators
    CMCON0 = 0b110;             // select mode 6 (CM = 110):
                                //   C1 -ref is C1IN-,
                                //   C1 +ref is C2IN+
                                //   C2 -ref is C2IN-,
                                //   C2 +ref is C2IN+,
                                //   both external outputs enabled,
                                //   both comparators on
    C1INV = 1;                  // C1 output inverted 
                                //  -> C1OUT = 1 if C1IN- > C2IN+
    C2INV = 1;                  // C2 output inverted 
                                //  -> C2OUT = 1 if C2IN- > C2IN+
                               
                                // enable comparator external outputs:
    TRISA2 = 0;                 //   C1OUT (RA2)
    TRISC4 = 0;                 //   C2OUT (RC4)
                                
    // configure interrupts
    PEIE = 1;                   // enable peripheral interrupts
    ei();                       // enable global interrupts
    
                                // enable comparator interrupts:
    CMCON0;                     //   read CMCON0 to clear mismatch
    C1IF = 0;                   //   clear interrupt flags
    C2IF = 0;
    C1IE = 1;                   //   set enable bits
    C2IE = 1;

                                    
    /*** Main loop ***/
    for (;;)
    {
        // continually copy shadow to port
        PORTC = sPORTC; 
    } 
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    // Service all triggered interrupt sources
    
    if (C1IF)
    {
        // Comparator 1 interrupt
        //   Triggered on any comparator 1 output change,
        //   caused by C1IN- input crossing C2IN+ (1.6 V) threshold
        //   
        CMCON0;                     // read CMCON0 to clear mismatch condition
        C1IF = 0;                   // clear interrupt flag
    
        // turn on LED 1 if C1IN- < C2IN+ 
        if (C1OUT)                  // if comparator output high (C1IN- > C2IN+)
        {         
            sPORTC &= ~(1<<nLED1);  //   turn off (shadow) LED 1 
        }
        else                        // else (C1IN- < C2IN+)
        {
            sPORTC |= 1<<nLED1;     //   turn on (shadow) LED 1 
        }
    }
    
    if (C2IF)
    {
        // Comparator 2 interrupt
        //   Triggered on any comparator 2 output change,
        //   caused by C2IN- input crossing C2IN+ (1.6 V) threshold
        //   
        CMCON0;                     // read CMCON0 to clear mismatch condition
        C2IF = 0;                   // clear interrupt flag
    
        // turn on LED 2 if C2IN- < C2IN+ 
        if (C2OUT)                  // if comparator output high (C2IN- > C2IN+)
        {         
            sPORTC &= ~(1<<nLED2);  //   turn off (shadow) LED 2 
        }
        else                        // else (C2IN- < C2IN+)
        {
            sPORTC |= 1<<nLED2;     //   turn on (shadow) LED 2 
        }
    }
}
