/************************************************************************
*                                                                       *
*   Filename:      MC_L6-Comp_2LEDs-HTC.c                               *
*   Date:          5/9/09                                               *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH C PRO v9.65PL1 Lite                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 6, example 6                                 *
*                                                                       *
*   Demonstrates use of programmable voltage reference                  *
*   to test that a signal is within limits                              *
*                                                                       *
*   Turns on Low LED  when CIN+ < 1.0 V (low light level)               *
*         or High LED when CIN+ > 2.0 V (high light level)              *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       CIN+  - voltage to be measured (e.g. pot output or LDR)         *
*       GP5   - "Low" LED                                               *
*       GP4   - "High" LED                                              *
*                                                                       *
************************************************************************/

#include <htc.h>

#define _XTAL_FREQ   4000000    // oscillator frequency for _delay()


/***** CONFIGURATION *****/
//      ext reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define nLO     5               // "Low" LED on GP5
#define nHI     4               // "High" LED on GP4


/***** MAIN PROGRAM *****/
void main()
{
    unsigned char   sGPIO;      // shadow copy of GPIO
    
    // Initialisation
    TRISIO = ~(1<<nLO|1<<nHI);  // configure LED pins as outputs
    
    // configure comparator
    CMCON = 0b110;              // select mode 6 (CM = 110):
                                //   +ref is CVref,
                                //   no external output,
                                //   comparator on
    CIS = 1;                    // -ref is CIN+
    CINV = 1;                   // inverted output 
                                // -> COUT = 1 if CIN+ > CVref
                                
    
    // Main loop
    for (;;)
    {
        sGPIO = 0;                  // start with both LEDs off
        
        // Test for low illumination
        // set low input threshold
        VRCON = 5;                  // select voltage: VR = 5,
        VRR = 1;                    //   low range -> CVref = 0.208*Vdd
        VREN = 1;                   // enable voltage reference
                                    // -> CVref = 1.04 V (if Vdd = 5.0 V) 
        __delay_us(10);             // wait 10us to settle
        // compare with input
        if (!COUT)                  // if CIN+ < CVref
            sGPIO |= 1<<nLO;        //   turn on Low LED
                    
        // Test for high illumination
        // set high input threshold
        VRCON = 5;                  // select voltage: VR = 5,
        VRR = 0;                    //   high range -> CVref = 0.406*Vdd
        VREN = 1;                   // enable voltage reference
                                    // -> CVref = 2.03 V (if Vdd = 5.0 V) 
        __delay_us(10);             // wait 10us to settle
        // compare with input
        if (COUT)                   // if CIN+ > CVref
            sGPIO |= 1<<nHI;        //   turn on High LED         
            
        // Display test results           
        GPIO = sGPIO;               // copy shadow to GPIO port
    }
}
