/************************************************************************
*                                                                       *
*   Filename:      MC_L6-Comp_4input-PCL.c                              *
*   Date:          25/10/09                                             *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: delay.h     : HI-TECH supplied delay code           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 6, example 8                                 *
*                                                                       *
*   Demonstrates dual comparator input multiplexing                     *
*                                                                       *
*   Turns on: LED 1 when C1IN+ > 1.5 V,                                 *
*             LED 2 when C1IN- > 1.5 V,                                 *
*             LED 3 when C2IN+ > 1.5 V,                                 *
*         and LED 4 when C2IN- > 1.5 V                                  *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       C1IN+ - input 1 (LDR/resistor divider)                          *
*       C1IN- - input 2 (LDR/resistor divider)                          *
*       C2IN+ - input 3 (LDR/resistor divider)                          *
*       C2IN- - input 4 (LDR/resistor divider)                          *
*       RC2   - indicator LED 1                                         *
*       RC3   - indicator LED 2                                         *
*       RC4   - indicator LED 3                                         *
*       RC5   - indicator LED 4                                         *
*                                                                       *
************************************************************************/

#include <htc.h>

#define XTAL_FREQ   4MHZ    // oscillator frequency for DelayUs()
#include "delay.h"          // defines DelayUs()


/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO & FCMDIS & IESODIS);

// Pin assignments
#define nLED1   2              // indicator LED 1 on RC2
#define nLED2   3              // indicator LED 2 on RC3
#define nLED3   4              // indicator LED 3 on RC4
#define nLED4   5              // indicator LED 4 on RC5


/***** MAIN PROGRAM *****/
void main()
{
    unsigned char   sPORTC;     // shadow copy of PORTC
    
    // Initialisation
    TRISC = 0b000011;           // configure RC0/C2IN+ and RC1/C2IN- as inputs,
                                //   rest of PORTC as outputs
                                //   (PORTA all inputs by default)
    
    // configure comparators
    CMCON0 = 0b010;             // select mode 2 (CM = 010):
                                //   C1 +ref is CVref,
                                //   C1 -ref is C1IN+ or C1IN- (selected by CIS),
                                //   C2 +ref is CVref,
                                //   C2 -ref is C2IN+ or C2IN- (selected by CIS),
                                //   no external outputs,
                                //   both comparators on
    C1INV = 1;                  // C1 output inverted 
                                // -> C1OUT = 1 if C1 -ref > CVref
    C2INV = 1;                  // C2 output inverted 
                                // -> C2OUT = 1 if C2 -ref > CVref                            
                                
    // configure voltage reference
    VRCON = 7;                  // select voltage: VR = 7,
    VRR = 1;                    //   low range -> CVref = 0.292*Vdd
    VREN = 1;                   // enable voltage reference
                                // -> CVref = 1.5 V (if Vdd = 5.0 V) 
    
    
    // Main loop
    for (;;)
    {
        sPORTC = 0;                // start with all LEDs off
        
        // test inputs 1 and 3
        CIS = 1;                    // select C1IN+ and C2IN+ pins as -refs
        DelayUs(10);                // wait 10us to settle
        if (C1OUT)                  // if C1IN+ > CVref
            sPORTC |= 1<<nLED1;     //   turn on LED 1
        if (C2OUT)                  // if C2IN+ > CVref
            sPORTC |= 1<<nLED3;     //   turn on LED 3
                    
        // test inputs 2 and 4
        CIS = 0;                    // select C1IN- and C2IN- pins as -refs
        DelayUs(10);                // wait 10us to settle
        if (C1OUT)                  // if C1IN- > CVref
            sPORTC |= 1<<nLED2;     //   turn on LED 2      
        if (C2OUT)                  // if C1IN- > CVref
            sPORTC |= 1<<nLED4;     //   turn on LED 4 
                        
        // display test results           
        PORTC = sPORTC;             // copy shadow to PORTC
    }
}
