/************************************************************************
*                                                                       *
*   Filename:      MC_L6-Comp_Interrupt-HTC.c                           *
*   Date:          30/8/09                                              *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH C PRO v9.65PL1 Lite                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 6, example 4                                 *
*                                                                       *
*   Demonstrates use of comparator interrupt                            *
*   (assumes hysteresis is used to reduce triggering)                   *
*                                                                       *
*   Turns on LED when voltage on CIN+ < 1.5 V                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       CIN+  - voltage to be measured (e.g. pot output or LDR)         *
*       COUT  - comparator output (fed back to input via resistor)      *
*       GP5   - indicator LED                                           *
*                                                                       *
************************************************************************/

#include <htc.h>


/***** CONFIGURATION *****/
//      ext reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define LED     GPIO5           // indicator LED on GP5
#define nLED    5               //   (port bit 5)


/***** GLOBAL VARIABLES *****/
unsigned char   sGPIO;          // shadow copy of GPIO


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    
    TRISIO = ~(1<<nLED);        // configure LED pin (only) as an output
    
    // configure comparator
    CMCON = 0b101;              // select mode 5 (CM = 101):
                                //   +ref is CVref,
                                //   external output enabled,
                                //   comparator on
    CIS = 1;                    // -ref is CIN+
    CINV = 1;                   // inverted output 
                                // -> COUT = 1 if CIN+ > CVref,
                                //    COUT pin enabled
                                
    TRIS2 = 0;                  // configure COUT (GP2) as an output
                                
    // configure voltage reference
    VRCON = 7;                  // select voltage: VR = 7,
    VRR = 1;                    //   low range -> CVref = 0.292*Vdd
    VREN = 1;                   // enable voltage reference
                                // -> CVref = 1.5 V (if Vdd = 5.0 V) 
    
    // configure interrupts
    PEIE = 1;                   // enable peripheral interrupts
    ei();                       // enable global interrupts
                                // enable comparator interrupt:
    CMCON;                      //   read CMCON to clear mismatch
    CMIF = 0;                   //   clear interrupt flag
    CMIE = 1;                   //   set enable bit
                                    
    // Main loop
    for (;;)
    {
        // continually copy shadow GPIO to port
        GPIO = sGPIO; 
    } 
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    // Service comparator interrupt
    //   Triggered on any comparator output change,
    //   caused by comparator input crossing 1.5 V threshold
    //   
    CMCON;                      // read CMCON to clear mismatch condition
    CMIF = 0;                   // clear interrupt flag
    
    // turn on LED if CIN+ < 1.5 V (using shadow register)
    if (COUT)                   // if comparator output high (CIN+ > 1.5 V)
    {         
        sGPIO &= ~(1<<nLED);    //   turn off LED
    }
    else                        // else (CIN+ < 1.5 V)
    {
        sGPIO |= 1<<nLED;       //   turn on LED
    }
}
