/************************************************************************
*                                                                       *
*   Filename:      MC_L6-Comp_Wakeup-HTC.c                              *
*   Date:          1/9/09                                               *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     12F629                                               *
*   Compiler:      HI-TECH C PRO v9.65PL1 Lite                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: stdmacros-HTC.h     : provides DelayS() macro       *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 6, example 5                                 *
*                                                                       *
*   Demonstrates wake-up on comparator change                           *
*                                                                       *
*   Turns on LED for 1 sec when comparator output changes               *
*   then sleeps until the next change                                   *
*   (threshold is 1.5 V, set by programmable voltage ref),              *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       CIN+  - voltage to be measured (e.g. pot output or LDR)         *
*       GP5   - indicator LED                                           *
*                                                                       *
************************************************************************/

#include <htc.h>

#define _XTAL_FREQ   4000000    // oscillator frequency for _delay()
#include "stdmacros-HTC.h"      // defines DelayS()


/***** CONFIGURATION *****/
//      ext reset, no code or data protect, no brownout detect,
//      no watchdog, power-up timer enabled, 4MHz int clock
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO);

// Pin assignments
#define LED     GPIO5           // indicator LED on GP5
#define nLED    5               //   (port bit 5)


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    
    // configure ports
    TRISIO = ~(1<<nLED);        // configure LED pin (only) as an output
    
    // configure comparator
    CMCON = 0b110;              // select mode 6 (CM = 110):
                                //   +ref is CVref,
                                //   no external output,
                                //   comparator on
    CIS = 1;                    // -ref is CIN+
    CINV = 0;                   // output not inverted
                                // -> COUT = 1 if CIN+ < CVref
                                
    // configure voltage reference
    VRCON = 7;                  // select voltage: VR = 7,
    VRR = 1;                    //   low range -> CVref = 0.292*Vdd
    VREN = 1;                   // enable voltage reference
                                // -> CVref = 1.5 V (if Vdd = 5.0 V) 
 
    // delay 10 ms to allow comparator and voltage ref to settle
    __delay_ms(10);
       
    // enable comparator interrupt (for wake on change)
    PEIE = 1;                   // enable peripheral interrupts
    CMIE = 1;                   // enable comparator interrupt

                                    
    // Main loop
    for (;;)
    {
        // turn off LED
        LED = 0;
        
        // enter sleep mode
        CMCON;                  // read CMCON to clear comparator mismatch
        CMIF = 0;               // clear comparator interrupt flag
        SLEEP();
        
        // turn on LED for 1 second
        LED = 1;                // turn on LED
        DelayS(1);              // delay 1 sec
     } 
}

