/************************************************************************
*                                                                       *
*   Filename:      MC_L7-Count_7seg_x1-int-HTC.c                        *
*   Date:          21/03/10                                             *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH C PRO v9.65PL1 Lite                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: stdmacros-HTC.h     : provides DelayS()             *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 7, example 1c                                *
*                                                                       *
*   Demonstrates use of a timer-based interrupt                         *
*   to update a single 7-segment display                                *
*                                                                       *
*   Single digit 7-segment display counts repeating 0 -> 9              *
*   1 second per count, with timing derived from int RC oscillator      *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       RA2, RC0-5 = 7-segment display (common cathode)                 *
*       RA0 - display enable (active high)                              *
*                                                                       *
************************************************************************/

#include <htc.h>

#define _XTAL_FREQ  4000000     // oscillator frequency for delay functions
#include "stdmacros-HTC.h"      // DelayS() - delay in seconds


/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO & FCMDIS & IESODIS);

// Pin assignments
#define nDISPLAY    0           // display enable on RA0


/***** PROTOTYPES *****/
void set7seg(char digit);       // display digit on 7-segment display (shadow)


/***** GLOBAL VARIABLES *****/
unsigned char   sPORTA;         // shadow registers: PORTA
unsigned char   sPORTC;         //                   PORTC
unsigned char   digit = 0;      // digit to be displayed by ISR


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // setup ports
    TRISA = 0;                      // configure PORTA and PORTC as all outputs
    TRISC = 0;
    // setup Timer0
    OPTION = 0b11000010;            // configure Timer0:
             //--0-----                 timer mode (T0CS = 0)
             //----0---                 prescaler assigned to Timer0 (PSA = 0)
             //-----010                 prescale = 8 (PS = 010)
                                    //  -> increment every 8 us
                                    //  -> TMR0 overflows every 2.048 ms
    // configure interrupts
    T0IE = 1;                       // enable Timer0 interrupt
    ei();                           // enable global interrupts
            
    // Main loop
    for (;;)
    {
        // display each digit from 0 to 9 for 1 sec
        for (digit = 0; digit < 10; digit++)
        {
            DelayS(1);              // delay 1 sec
        }  
    }      
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    // *** Service Timer0 interrupt
    //  TMR0 overflows every 2.048 ms
    //  (only Timer0 interrupts are enabled)
    //
    T0IF = 0;                       // clear interrupt flag
    
    // display digit (using shadow registers)
    set7seg(digit);                 // output digit
    sPORTA |= 1 << nDISPLAY;        // enable display
    
    // copy shadow regs to ports        
    PORTA = sPORTA;             
    PORTC = sPORTC;    
}


/***** FUNCTIONS *****/

/***** Display digit on 7-segment display (shadow) *****/
void set7seg(char digit)
{
    // Lookup pattern table for 7 segment display on PORTA
    const char pat7segA[10] = {
        // RA2 = G
        0b000000,   // 0
        0b000000,   // 1
        0b000100,   // 2
        0b000100,   // 3
        0b000100,   // 4
        0b000100,   // 5
        0b000100,   // 6
        0b000000,   // 7
        0b000100,   // 8
        0b000100    // 9
    }; 

    // Lookup pattern table for 7 segment display on PORTC
    const char pat7segC[10] = {
        // RC5:0 = ABCDEF
        0b111111,   // 0
        0b011000,   // 1
        0b110110,   // 2
        0b111100,   // 3
        0b011001,   // 4
        0b101101,   // 5
        0b101111,   // 6
        0b111000,   // 7
        0b111111,   // 8
        0b111101    // 9
    }; 
    
    // lookup pattern bits and write to shadow registers
    sPORTA = pat7segA[digit];     
    sPORTC = pat7segC[digit];
}