/************************************************************************
*                                                                       *
*   Filename:      MC_L7-Count_7seg_x3-HTC.c                            *
*   Date:          21/2/10                                              *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH C PRO v9.65PL1 Lite                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: stdmacros-HTC.h     : provides DelayS()             *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 7, example 2                                 *
*                                                                       *
*   Demonstrates use of timer-based interrupt-driven multiplexing       *
*   to drive multiple 7-seg displays                                    *
*                                                                       *
*   3 digit 7-segment LED display: 1 digit minutes, 2 digit seconds     *
*   counts in seconds 0:00 to 9:59 then repeats,                        *
*   with timing derived from int 4 MHz oscillator                       *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       RA2, RC0-5 = 7-segment display (common cathode)                 *
*       RA4 - minutes enable (active high)                              *
*       RA1 - tens enable                                               *
*       RA0 - ones enable                                               *
*                                                                       *
************************************************************************/

#include <htc.h>

#define _XTAL_FREQ  4000000     // oscillator frequency for delay functions
#include "stdmacros-HTC.h"      // DelayS() - delay in seconds


/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO & FCMDIS & IESODIS);

// Pin assignments
#define nMINS   4           // minutes enable on RA4
#define nTENS   1           // tens enable on RA1
#define nONES   0           // ones enable on RA0


/***** PROTOTYPES *****/
void set7seg(char digit);       // display digit on 7-segment display (shadow)


/***** GLOBAL VARIABLES *****/
unsigned char   sPORTA;         // shadow registers: PORTA
unsigned char   sPORTC;         //                   PORTC
unsigned char   mins = 0;     	// time counters (displayed by ISR)
unsigned char	secs = 0;


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    // setup ports
    TRISA = 0;                      // configure PORTA and PORTC as all outputs
    TRISC = 0;
    // setup Timer0
    OPTION = 0b11000010;            // configure Timer0:
             //--0-----                 timer mode (T0CS = 0)
             //----0---                 prescaler assigned to Timer0 (PSA = 0)
             //-----010                 prescale = 8 (PS = 010)
                                    //  -> increment every 8 us
                                    //  -> TMR0 overflows every 2.048 ms
    // configure interrupts
    T0IE = 1;                       // enable Timer0 interrupt
    ei();                           // enable global interrupts
            
    // Main loop
    for (;;)
    {
        // count minutes:seconds from 0:00 to 9:59
        // (displayed by ISR)
        for (mins = 0; mins < 10; mins++)
        {
            for (secs = 0; secs < 60; secs++)
            {
                DelayS(1);              // delay 1 sec
            }
        }  
    }      
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    static unsigned char    mpx_cnt = 0;    // multiplex counter
    
    // *** Service Timer0 interrupt
    //  TMR0 overflows every 2.048 ms
    //  (only Timer0 interrupts are enabled)
    //
    T0IF = 0;                       // clear interrupt flag
    
    // Display current count on 3 x 7-segment displays
    //   mpx_cnt determines current digit to diplay
    //
    switch (mpx_cnt)
    {
        case 0: 
            set7seg(secs%10);               // output ones digit  
            sPORTA |= 1 << nONES;           // enable ones display
            break;
        case 1:
            set7seg(secs/10);               // output tens digit  
            sPORTA |= 1 << nTENS;           // enable tens display
            break;
        case 2:
            set7seg(mins);                  // output minutes digit
            sPORTA |= 1 << nMINS;           // enable minutes display
            break;
    }
    // Increment mpx_cnt, to select next digit for next time
    mpx_cnt++;
    if (mpx_cnt == 3)       // reset count if at end of digit sequence
        mpx_cnt = 0;
            
    // copy shadow regs to ports        
    PORTA = sPORTA;             
    PORTC = sPORTC;    
}


/***** FUNCTIONS *****/

/***** Display digit on 7-segment display (shadow) *****/
void set7seg(char digit)
{
    // Lookup pattern table for 7 segment display on PORTA
    const char pat7segA[10] = {
        // RA2 = G
        0b000000,   // 0
        0b000000,   // 1
        0b000100,   // 2
        0b000100,   // 3
        0b000100,   // 4
        0b000100,   // 5
        0b000100,   // 6
        0b000000,   // 7
        0b000100,   // 8
        0b000100    // 9
    }; 

    // Lookup pattern table for 7 segment display on PORTC
    const char pat7segC[10] = {
        // RC5:0 = ABCDEF
        0b111111,   // 0
        0b011000,   // 1
        0b110110,   // 2
        0b111100,   // 3
        0b011001,   // 4
        0b101101,   // 5
        0b101111,   // 6
        0b111000,   // 7
        0b111111,   // 8
        0b111101    // 9
    }; 
    
    // lookup pattern bits and write to shadow registers
    sPORTA = pat7segA[digit];     
    sPORTC = pat7segC[digit];
}