/************************************************************************
*                                                                       *
*   Filename:      MC_L8-ADC_dec-out-HTC.c                              *
*   Date:          2/4/10                                               *
*   File Version:  1.1                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH C v9.70 (Lite mode)                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 8, example 6                                 *
*                                                                       *
*   Displays ADC output in decimal on 2x7-segment LED displays          *
*                                                                       *
*   Continuously samples analog input, scales result to 0 - 99          *
*   and displays as 2 x dec digits on multiplexed 7-seg displays        *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       AN0         = voltage to be measured (e.g. pot or LDR)          *
*       RA5, RC0-5  = 7-segment display bus (common cathode)            *
*       RA4         = tens enable (active high)                         *
*       RA1         = ones enable                                       *
*                                                                       *
************************************************************************/

#include <htc.h>

#define _XTAL_FREQ  4000000     // oscillator frequency for __delay_us()


/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, no power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTDIS & INTIO & FCMDIS & IESODIS);

// Pin assignments
#define nTENS_EN    4           // tens enable on RA4
#define nONES_EN    1           // ones enable on RA1


/***** PROTOTYPES *****/
void set7seg(char digit);       // display digit on 7-segment display (shadow)


/***** GLOBAL VARIABLES *****/
unsigned char   sPORTA;         // shadow registers: PORTA
unsigned char   sPORTC;         //                   PORTC

                                // current result in decimal (displayed by ISR):
unsigned char   tens = 0;       //   tens
unsigned char   ones = 0;       //   ones


/***** MAIN PROGRAM *****/
void main()
{
    unsigned char    adc_dec;        // scaled ADC output (0-99)
    
    // Initialisation
    
    // configure ports
    TRISC = 0;                  // PORTC is all outputs
    TRISA = 1<<0;               // configure RA0/AN0 (only) as an input
    ANSEL = 1<<0;               // make only AN0 analog
    CMCON0 = 7;                 // disable comparators  (CM = 7 -> both comparators off) 
    
    // configure Timer0
    OPTION = 0b11000010;            // configure Timer0:
             //--0-----                 timer mode (T0CS = 0)
             //----0---                 prescaler assigned to Timer0 (PSA = 0)
             //-----010                 prescale = 8 (PS = 010)
                                    //  -> increment every 8 us
                                    //  -> TMR0 overflows every 2.048 ms    
                                    
    // configure ADC     
    ADCON1 = 0b00010000;
             //-001----          Tad = 8*Tosc (ADCS = 001) = 2.0 us (with Fosc = 4 MHz) 
    ADCON0 = 0b00000001;       
             //0-------          MSB of result in ADRESH<7> (ADFM = 0)
             //-0------          voltage reference is Vdd (VCFG = 0)
             //---000--          select channel AN0 (CHS = 000) 
             //-------1          turn ADC on (ADON = 1)
             

    // configure interrupts
    T0IE = 1;                       // enable Timer0 interrupt
    ei();                           //  and global interrupts
    
            
    // Main loop
    for (;;)
    {
        // sample analog input
        __delay_us(10);         // wait 10 us for acquisition time
        GODONE = 1;             // start conversion
        while (GODONE)          // wait until done
            ;
            
        // scale 8-bit ADC result to 0-99
        adc_dec = (unsigned)ADRESH * 100/256;
        
        // disable interrupts during display variable update
        // (stops ISR attempting to display out-of-range intermediate results)
        di();
                        
            // extract digits of scaled result for ISR to display
            ones = (unsigned)adc_dec%10;   
            tens = (unsigned)adc_dec/10;  
        
            // copy shadow regs (updated by ISR) to ports        
            PORTA = sPORTA;             
            PORTC = sPORTC;   
        
        // re-enable interrupts
        ei(); 
    }      
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    static unsigned char    mpx_cnt = 0;    // multiplex counter
    
    // *** Service Timer0 interrupt
    //  TMR0 overflows every 2.048 ms
    //  (only Timer0 interrupts are enabled)
    //
    T0IF = 0;                       // clear interrupt flag
    
    // Display current ADC result (in hex) on 3 x 7-segment displays
    //   mpx_cnt determines current digit to diplay
    //
    switch (mpx_cnt)
    {
        case 0: 
            set7seg(ones);                  // output ones digit  
            sPORTA |= 1 << nONES_EN;        // enable ones display
            break;
        case 1:
            set7seg(tens);                  // output tens digit  
            sPORTA |= 1 << nTENS_EN;        // enable tens display
            break;
    }
    // Increment mpx_cnt, to select next digit for next time
    mpx_cnt++;
    if (mpx_cnt == 2)       // reset count if at end of digit sequence
        mpx_cnt = 0;
            

}


/***** FUNCTIONS *****/

/***** Display digit on 7-segment display (shadow) *****/
void set7seg(char digit)
{
    // Lookup pattern table for 7 segment display on PORTA
    const char pat7segA[16] = {
        // RA5 = G
        0b000000,   // 0
        0b000000,   // 1
        0b100000,   // 2
        0b100000,   // 3
        0b100000,   // 4
        0b100000,   // 5
        0b100000,   // 6
        0b000000,   // 7
        0b100000,   // 8
        0b100000,   // 9
    }; 

    // Lookup pattern table for 7 segment display on PORTC
    const char pat7segC[16] = {
        // RC5:0 = ABCDEF
        0b111111,   // 0
        0b011000,   // 1
        0b110110,   // 2
        0b111100,   // 3
        0b011001,   // 4
        0b101101,   // 5
        0b101111,   // 6
        0b111000,   // 7
        0b111111,   // 8
        0b111101,   // 9
    }; 
    
    // lookup pattern bits and write to shadow registers
    sPORTA = pat7segA[digit];     
    sPORTC = pat7segC[digit];
}