/************************************************************************
*                                                                       *
*   Filename:      MC_L9-Count_7seg_x1-PCL                              *
*   Date:          13/3/11                                              *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 9, example 4                                 *
*                                                                       *
*   Demonstrates use of Timer1 LP oscillator and interrupt              *
*   to implement a single-digit seconds counter,                        *
*   shown on a 7-segment LED display                                    *
*                                                                       *
*   Single digit 7-segment LED display counts repeating 0 -> 9          *
*   1 count per second, with timing derived from 32.768 kHz crystal     *
*   driven by Timer1 oscillator                                         *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       RA0-2, RC2-5 - 7-segment display bus (common cathode)           *
*       OSC1, OSC2   - 32.768 kHz crystal                               *
*                                                                       *
************************************************************************/

#include <htc.h>
                            

/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLREN & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO & FCMDIS & IESODIS);


/***** PROTOTYPES *****/
void set7seg(char digit);           // display digit on 7-segment display


/***** GLOBAL VARIABLES *****/
volatile unsigned char  t1_secs = 0;    // seconds count (even only),
                                        //  updated by Timer1 interrupt


/***** MAIN PROGRAM *****/
void main()
{
    unsigned char   digit;          // digit to display
    
    // Initialisation
    
    // configure ports
    PORTA = 0;                      // start with PORTA and PORTC clear 
    PORTC = 0;                      //   (all LED segments off)
    TRISA = 0;                      // configure PORTA and PORTC as all outputs
    TRISC = 0;     
    
    // configure timers
    T1CON = 0b00001010;             // configure Timer1:
            //-0------                  gate disabled (TMR1GE = 0)
            //--00----                  prescale = 1 (T1CKPS = 00)
            //----1---                  LP oscillator enabled (T1OSCEN = 1)
            //------1-                  external clock (TMR1CS = 1)
            //-------0                  disable Timer1 (TMR1ON = 0)
                                    //  -> increment TMR1 at 32.768 kHz
    TMR1H = 0;                      // clear TMR1 (start counting from 0)
    TMR1L = 0;       
    TMR1ON = 1;                     // start Timer1   
    TMR1IE = 1;                     // enable Timer1 interrupt
    
    // enable interrupts
    PEIE = 1;                       // enable peripheral 
    ei();                           //   and global interrupts    
           
                 
    // Main loop  
    for (;;)
    {
        // calculate digit to display
        digit = t1_secs;            // get seconds count (even only) 
                                    //   (maintained by Timer1 interrupt)
        if (TMR1H & 1<<7)           // if TMR1H<7> is set, we are in an odd second
            ++digit;                //   so add one to display digit
                                        
        // display current seconds count
        set7seg(digit);             // output digit
    } 
}


/***** INTERRUPT SERVICE ROUTINE *****/
void interrupt isr(void)
{
    // *** Service Timer1 interrupt
    //  TMR1 overflows every 2 s 
    //
    //  Handles background time keeping:
    //  Each interrupt advances current count by 2 secs    
    //
    //  (only Timer1 interrupts are enabled)
    //
    TMR1IF = 0;                     // clear interrupt flag
    
    // increment seconds count by 2
    t1_secs += 2;
    if (t1_secs == 10)          // when count reaches 10, 
        t1_secs = 0;            //   reset it to 0
}


/***** FUNCTIONS *****/

/***** Display digit on 7-segment display *****/
void set7seg(char digit)
{
    // Lookup pattern table for 7 segment display on PORTA
    const char pat7segA[10] = {
        // RA2:0 = EFG
        0b000110,   // 0
        0b000000,   // 1
        0b000101,   // 2
        0b000001,   // 3
        0b000011,   // 4
        0b000011,   // 5
        0b000111,   // 6
        0b000000,   // 7
        0b000111,   // 8
        0b000011    // 9
    }; 

    // Lookup pattern table for 7 segment display on PORTC
    const char pat7segC[10] = {
        // RC5:2 = ABCD
        0b111100,   // 0
        0b011000,   // 1
        0b110100,   // 2
        0b111100,   // 3
        0b011000,   // 4
        0b101100,   // 5
        0b101100,   // 6
        0b111000,   // 7
        0b111100,   // 8
        0b111100    // 9
    }; 
    
    // lookup pattern bits and write to port registers
    PORTA = pat7segA[digit];     
    PORTC = pat7segC[digit];
}