/************************************************************************
*                                                                       *
*   Filename:      MC_L9-Flash+PB_LED-PCL                               *
*   Date:          9/3/11                                               *
*   File Version:  1.2                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH PICC-Lite v9.60PL2                           *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 9, example 1                                 *
*                                                                       *
*   Demonstrates use of Timer1 to maintain timing of background actions *
*   while performing other actions in response to changing inputs       *
*                                                                       *
*   One LED simply flashes at 1 Hz (50% duty cycle).                    *
*   The other LED is only lit when the pushbutton is pressed            *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       RC0 - "button pressed" indicator LED                            *
*       RC1 - flashing LED                                              *
*       RA3 - pushbutton (active low)                                   *
*                                                                       *
************************************************************************/

#include <htc.h>


/***** CONFIGURATION *****/
//  int reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLRDIS & UNPROTECT & BORDIS & WDTDIS & PWRTEN & INTIO & FCMDIS & IESODIS);

// Pin assignments
#define nB_LED  0           // "button pressed" indicator LED on RC0
#define nF_LED  1           // flashing LED on RC1
#define BUTTON  RA3         // pushbutton (active low)


/***** CONSTANTS *****/
#define FlashMS 500                     // LED flash toggle time in milliseconds
#define InitT1  65536-FlashMS*1000/8    // Initial value to load into TMR1 to generate
                                        //   FlashMS delay (assuming 8 us/tick)

/***** GLOBAL VARIABLES *****/
unsigned char   sPORTC;                 // shadow copy of PORTC


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    
    // configure ports
    PORTC = 0;                      // start with PORTC clear (all LEDs off)
    sPORTC = 0;                     //   and update shadow
    TRISC = 0;                      // configure all PORTC pins as outputs
    
    // configure timers
    T1CON = 0b00110000;             // configure Timer1:
            //-0------                  gate disabled (TMR1GE = 0)
            //--11----                  prescale = 8 (T1CKPS = 11)
            //----0---                  LP oscillator disabled (T1OSCEN = 0)
            //------0-                  internal clock (TMR1CS = 0)
            //-------0                  disable Timer1 (TMR1ON = 0)  
                                    //  -> increment TMR1 every 8 us
    TMR1IF = 0;                     //  clear overflow flag  
    
                 
    // Main loop  
    for (;;)
    {
        // delay ~500ms while checking for button press
        TMR1ON = 0;                 // stop Timer1
        TMR1H = InitT1 >> 8;        // load TMR1 with initial value (InitT1)
        TMR1L = InitT1 & 0xFF;      //   to generate 500 ms count 
        TMR1ON = 1;                 // start Timer1
        
        while (!TMR1IF)             // repeat until Timer1 overflows
        {
            // respond to button press
            if (!BUTTON)                    // if button pressed (low)
                sPORTC |= 1<<nB_LED;        //   turn on indicator LED
            else                            // else
                sPORTC &= ~(1<<nB_LED);     //   turn it off
            
            // copy shadow register to port    
            PORTC = sPORTC;
        }  
        // (end 500 ms delay)
        
        TMR1IF = 0;                     // clear overflow flag for next time

        // flash LED
        sPORTC ^= 1<<nF_LED;               
    } 
}
