/************************************************************************
*                                                                       *
*   Filename:      MC_L9-TMR1_gate-HTC                                  *
*   Date:          21/3/11                                              *
*   File Version:  1.0                                                  *
*                                                                       *
*   Author:        David Meiklejohn                                     *
*   Company:       Gooligum Electronics                                 *
*                                                                       *
*************************************************************************
*                                                                       *
*   Architecture:  Midrange PIC                                         *
*   Processor:     16F684                                               *
*   Compiler:      HI-TECH C v9.81 (Lite mode)                          *
*                                                                       *
*************************************************************************
*                                                                       *
*   Files required: none                                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Description:    Lesson 9, example 6                                 *
*                                                                       *
*   Demonstrates use of Timer1 gate control                             *
*   to measure the pulse width of a digital signal on /T1G,             *
*   scaled and displayed as a single hex digit                          *
*                                                                       *
*   Timer1 is used to time (in microseconds) every high pulse on /T1G.  *
*   Result is divided by 256 and displayed in hex on a single-digit     *
*   7-segment LED display.                                              *
*   Time base is internal RC oscillator.                                *
*                                                                       *
*************************************************************************
*                                                                       *
*   Pin assignments:                                                    *
*       RA0-2, RC2-5 - 7-segment display bus (common cathode)           *
*       /T1G (=RA4)  - signal to measure pulse width of                 *
*                      (active high, 4 ms max)                          *    
*                                                                       *
************************************************************************/

#include <htc.h>
                            

/***** CONFIGURATION *****/
//  ext reset, no code or data protect, no brownout detect,
//  no watchdog, power-up timer, int clock with I/O,
//  no failsafe clock monitor, two-speed start-up disabled 
__CONFIG(MCLRE_ON & CP_OFF & CPD_OFF & BOREN_OFF & WDTE_OFF & PWRTE_ON &
         FOSC_INTOSCIO & FCMEN_OFF & IESO_OFF);

// Pin assignments
#define T1G     RA4         // Timer1 gate input (configured as active high)
#define nT1G    4           //   (pin 4)


/***** PROTOTYPES *****/
void set7seg(char digit);           // display digit on 7-segment display


/***** MAIN PROGRAM *****/
void main()
{
    // Initialisation
    
    // configure ports
    PORTA = 0;                      // start with PORTA and PORTC clear 
    PORTC = 0;                      //   (all LED segments off)
    TRISC = 0;                      // configure PORTA and PORTC as all outputs
    TRISA = 1<<nT1G;                //   except T1G/RA4 input 
    ANSEL = 0;                      // no analog inputs            
    
    // configure timers
    T1CON = 0b11000001;             // configure Timer1:
            //1-------                  gate is active high (T1GINV = 1)
            //-1------                  gate enabled (TMR1GE = 1)
            //--00----                  prescale = 1 (T1CKPS = 00)
            //----0---                  LP oscillator disabled (T1OSCEN = 0)
            //------0-                  internal clock (TMR1CS = 0)
            //-------1                  enable Timer1 (TMR1ON = 1)
                                    //  -> increment TMR1 every 1 us,
    T1GSS = 1;                      //     gated by active high on /T1G
           
                 
    // Main loop  
    for (;;)
    {
        // Measure width of pulses on /T1G input
        // (assume /T1G is low at start of loop)
        
        // clear Timer1
        TMR1 = 0;              
        
        // wait for one full high pulse 
        while (!T1G)                // wait for T1G to go high
            ;
        while (T1G)                 // wait for T1G to go low
            ;   
        
        // display scaled Timer1 count (divide by 256)         
        set7seg(TMR1/256 & 0x0f);    // display low nybble of TMR1/256
    } 
}


/***** FUNCTIONS *****/

/***** Display digit on 7-segment display *****/
void set7seg(char digit)
{
    // Lookup pattern table for 7 segment display on PORTA
    const char pat7segA[16] = {
        // RA2:0 = EFG
        0b000110,   // 0
        0b000000,   // 1
        0b000101,   // 2
        0b000001,   // 3
        0b000011,   // 4
        0b000011,   // 5
        0b000111,   // 6
        0b000000,   // 7
        0b000111,   // 8
        0b000011,   // 9
        0b000111,   // A
        0b000111,   // b
        0b000110,   // C
        0b000101,   // d
        0b000111,   // E
        0b000111    // F 
    }; 

    // Lookup pattern table for 7 segment display on PORTC
    const char pat7segC[16] = {
        // RC5:2 = ABCD
        0b111100,   // 0
        0b011000,   // 1
        0b110100,   // 2
        0b111100,   // 3
        0b011000,   // 4
        0b101100,   // 5
        0b101100,   // 6
        0b111000,   // 7
        0b111100,   // 8
        0b111100,   // 9
        0b111000,   // A
        0b001100,   // b
        0b100100,   // C
        0b011100,   // d
        0b100100,   // E
        0b100000    // F
    }; 
    
    // lookup pattern bits and write to port registers
    PORTA = pat7segA[digit];     
    PORTC = pat7segC[digit];
}